--
-- Extended TipTrigger (ETT)
-- Base by GIANTS
-- Extended by Blacky_BPG
-- Version 4.0.0
-- Last Change: 23.12.2013 by Blacky_BPG
-- 

TipTrigger.load = function(self,id)
	self.rootNode = id;
	self.triggerId = Utils.indexToObject(id, getUserAttribute(id, "triggerIndex"));
	if self.triggerId == nil then
		self.triggerId = id;
	end
	addTrigger(self.triggerId, "triggerCallback", self);

	local isWheat = Fillable.fillTypeNameToDesc["wheat_windrow"];
	local isRye = Fillable.fillTypeNameToDesc["rye_windrow"];
	local isOat = Fillable.fillTypeNameToDesc["oat_windrow"];
	local isDinkel = Fillable.fillTypeNameToDesc["dinkel_windrow"];

	self.appearsOnPDA = Utils.getNoNil(getUserAttribute(id, "appearsOnPDA"), false);
	self.isFarmTrigger = Utils.getNoNil(getUserAttribute(id, "isFarmTrigger"), false);
	self.isBGA = Utils.getNoNil(getUserAttribute(id, "isBGA"), false);
	self.stationName = Utils.getNoNil(getUserAttribute(id, "stationName"), "Station");
	self.stationIdx = Utils.getNoNil(getUserAttribute(id, "stationIdx"), 1);

	self.isAreaTrigger = Utils.getNoNil(getUserAttribute(id, "isAreaTrigger"), false);
	if self.isAreaTrigger then
		self.triggerStartId = getChildAt(id, 0);
		self.triggerEndId = getChildAt(id, 1);
		self.triggerWidth = Utils.getNoNil(getUserAttribute(id, "triggerWidth"), 3);
	else
		self.triggerStartId = self.triggerId;
	end

	self.triggerTipWidth = Utils.getNoNil(getUserAttribute(id, "triggerTipWidth"), math.huge);
	self.changeType = nil;
	local changeType = getUserAttribute(id, "changeType");
	if changeType ~= nil then
		self.changeType = Fillable.fillTypeNameToInt[changeType];
	end;

	if self.priceMultipliers == nil or self.acceptedFillTypes == nil then
		self.acceptedFillTypes = {};
		self.priceMultipliers = {};
		self.priceMultipliersBackup = {};
		self.maxAmountAllowed = {};
		self.convertFruitsTo = {};

		local fillTypes = getUserAttribute(id, "fillTypes");
		if fillTypes == nil then
			fillTypes = getUserAttribute(id, "fruitTypes");
		end;
		local addRye = false;
		local addOat = false;
		local addDinkel = false;
		if string.find(fillTypes, "wheat_windrow") then
			if isRye ~= nil and not string.find(fillTypes, "rye_windrow") then
				fillTypes = fillTypes .. " rye_windrow";
				addRye = true;
			end;
			if isOat ~= nil and not string.find(fillTypes, "oat_windrow") then
				fillTypes = fillTypes .. " oat_windrow";
				addOat = true;
			end;
			if isDinkel ~= nil and not string.find(fillTypes, "dinkel_windrow") then
				fillTypes = fillTypes .. " dinkel_windrow";
				addDinkel = true;
			end;
		end;
		local priceMultipliersString = getUserAttribute(id, "priceMultipliers");
		if priceMultipliersString ~= nil then
			if addRye then
				priceMultipliersString = priceMultipliersString .. " 1";
			end;
			if addOat then
				priceMultipliersString = priceMultipliersString .. " 1";
			end;
			if addDinkel then
				priceMultipliersString = priceMultipliersString .. " 1";
			end;
		end;
		local mAA = getUserAttribute(id, "maxAmountAllowed");
		local cFT = getUserAttribute(id, "convertFruitsTo");
		if fillTypes ~= nil then
			local types = Utils.splitString(" ", fillTypes);

			local multipliers = {};
			local amounts = {};
			local converts =  {};
			if mAA ~= nil then
				amounts = Utils.splitString(" ", mAA);
			end;
			local converts = Fillable.fillTypeNameToDesc[cFT];
			if converts == nil then
				converts = {};
				converts.index = nil;
			end;

			if priceMultipliersString ~= nil then
				multipliers = Utils.splitString(" ", priceMultipliersString);
			elseif not self.isFarmTrigger and self.defaultPriceMultiplier == nil then
				print("Error: Missing priceMultipliers string user attribute for TipTrigger "..getName(id));
			end;
			for k,v in pairs(types) do
				local desc = Fillable.fillTypeNameToDesc[v];
				if desc ~= nil then
					self.acceptedFillTypes[desc.index] = true;
					self.priceMultipliers[desc.index] = tonumber(multipliers[k]);
					self.priceMultipliersBackup[desc.index] = tonumber(multipliers[k]);
					self.maxAmountAllowed[desc.index] = tonumber(amounts[k]);
					self.convertFruitsTo[desc.index] = converts.index;
					if isRye ~= nil and desc.index == isRye.index then
						self.convertFruitsTo[desc.index] = isWheat.index;
					end;
					if isOat ~= nil and desc.index == isOat.index then
						self.convertFruitsTo[desc.index] = isWheat.index;
					end;
					if isDinkel ~= nil and desc.index == isDinkel.index then
						self.convertFruitsTo[desc.index] = isWheat.index;
					end;
					if self.maxAmountAllowed[desc.index] == nil then
						self.maxAmountAllowed[desc.index] = -1;
					end;
					if self.convertFruitsTo[desc.index] == nil then
						self.convertFruitsTo[desc.index] = desc.index;
					end;
					if self.priceMultipliers[desc.index] == nil then
						self.priceMultipliers[desc.index] = Utils.getNoNil(self.defaultPriceMultiplier, 0);
						if not self.isFarmTrigger and self.defaultPriceMultiplier == nil then
							print("Error: "..k.."th priceMultiplier is invalid in TipTrigger "..getName(id));
						end;
					end;
				end;
			end;
		end;
	end

	local movingIndex = getUserAttribute(id, "movingIndex");
	if movingIndex ~= nil then
		local movingRootNode = id;
		if self.triggerId == id then
			movingRootNode = getParent(id);
		end
		self.movingId = Utils.indexToObject(movingRootNode, movingIndex);
		if self.movingId ~= nil then
			self.moveMinY = Utils.getNoNil(getUserAttribute(id, "moveMinY"), 0);
			self.moveMaxY = Utils.getNoNil(getUserAttribute(id, "moveMaxY"), 0);
			self.moveScale = Utils.getNoNil(getUserAttribute(id, "moveScale"), 0.001)*0.01;
			self.moveBackScale = (self.moveMaxY-self.moveMinY)/Utils.getNoNil(getUserAttribute(id, "moveBackTime"), 10000);
		end;
	end;
	local rollIndex = getUserAttribute(id, "rollIndex");
	self.rolls = {};
	if rollIndex ~= nil then
		local rollRootNode = id;
		if self.triggerId == id then
			rollRootNode = getParent(id);
		end
		local rollIdx = Utils.indexToObject(rollRootNode, rollIndex);
		if rollIdx ~= nil then
			local numChildren = Utils.getNoNil(getNumOfChildren(rollIdx),0);
			for i=1, numChildren do
				local rolle = getChildAt(rollIdx, i-1);
				if rolle ~= nil then
					table.insert(self.rolls, rolle);
				end;
			end;
		end;
	end;
	local rollIndex2 = getUserAttribute(id, "rollIndex2");
	self.rolls2 = {};
	if rollIndex2 ~= nil then
		local rollRootNode = id;
		if self.triggerId == id then
			rollRootNode = getParent(id);
		end
		local rollIdx2 = Utils.indexToObject(rollRootNode, rollIndex2);
		if rollIdx2 ~= nil then
			local numChildren = Utils.getNoNil(getNumOfChildren(rollIdx2),0);
			for i=1, numChildren do
				local rolle = getChildAt(rollIdx2, i-1);
				if rolle ~= nil then
					table.insert(self.rolls2, rolle);
				end;
			end;
		end;
	end;
	local rollSpeed = getUserAttribute(id, "rollSpeed");
	self.rollSpeed = -0.01;
	if rollSpeed ~= nil then
		self.rollSpeed = math.rad(rollSpeed/1000);
	end;

	local soundIndex = getUserAttribute(id, "soundIndex");
	if soundIndex ~= nil then
		local soundIndexNode = id;
		if self.triggerId == id then
			soundIndexNode = getParent(id);
		end
		self.soundIndex = Utils.indexToObject(soundIndexNode, soundIndex);
		if self.soundIndex ~= nil then
			setVisibility(self.soundIndex, false);
		end;
	end;
    if self.appearsOnPDA then
	   local x, _, z = getWorldTranslation(id);
	   local iconSize = g_currentMission.missionPDA.pdaMapWidth / 15;
	   self.mapHotspot = g_currentMission.missionPDA:createMapHotspot("TipPlace", Utils.getFilename("$dataS2/missions/hud_pda_spot_tipPlaceGold.png", self.baseDirectory), x, z, iconSize, iconSize * (4 / 3), false, false, false, 0, true);
    end;

    self.oldHourPrice = 0;

    self.isEnabled = true;

    self.TipTriggerDirtyFlag = self:getNextDirtyFlag();

    return true;
end;
TipTrigger.update = function(self,dt)
	if self.movingId ~= nil then
		local x,y,z = getTranslation(self.movingId);
		local newY = math.max(y-dt*self.moveBackScale, self.moveMinY);
		setTranslation(self.movingId, x, newY, z);
		if self.rolls ~= nil and table.getn(self.rolls) > 0 and newY > self.moveMinY then
			for i=1, table.getn(self.rolls) do
				rotate(self.rolls[i], self.rollSpeed*dt, 0,0);
			end;
		end;
		if self.rolls2 ~= nil and table.getn(self.rolls2) > 0 and newY > self.moveMinY then
			for i=1, table.getn(self.rolls2) do
				rotate(self.rolls2[i], (self.rollSpeed/2)*dt, 0,0);
			end;
		end;
		if self.soundIndex ~= nil then
			if newY > self.moveMinY then
				if not getVisibility(self.soundIndex) then
					setVisibility(self.soundIndex, true);
				end;
			else
				if getVisibility(self.soundIndex) then
					setVisibility(self.soundIndex, false);
				end;
			end;
		end;
	end;
	if self.changeType ~= nil then
		if Fillable.fillTypeIndexToDesc[self.changeType] ~= nil then
			local hourPrice = Fillable.fillTypeIndexToDesc[self.changeType].pricePerLiter;
			if self.oldHourPrice ~= hourPrice then
				self.oldHourPrice = hourPrice;
				for k, v in pairs(self.priceMultipliersBackup) do
					self.priceMultipliers[k] = v * (hourPrice * hourPrice);
				end;
			end;
		end;
	end;
end;
TipTrigger.updateTrailerTipping = function(self,trailer, fillDelta, fillType)
	local fillDeltaOld = fillDelta;
	local desc = Fillable.fillTypeIndexToDesc[fillType];
	local fType = fillType;
	if desc ~= nil and desc.name ~= nil then
		if self.stationIdx > 1 then
			fType = Fillable.fillTypeNameToInt[desc.name..self.stationIdx];
		end;
	end;
	local mAmount = 0;
	if self.maxAmountAllowed ~= nil and self.maxAmountAllowed[fillType] ~= nil and self.maxAmountAllowed[fillType] > 1 then
		mAmount = self.maxAmountAllowed[fillType];
		if mAmount == nil then
			mAmount = 0;
		end;
	end;
	local cFruitsTo = fillType;
	if self.convertFruitsTo ~= nil then
		cFruitsTo = self.convertFruitsTo[fillType];
	else
		self.convertFruitsTo = {};
		self.convertFruitsTo[fillType] = fillType;
	end;
	if trailer.fillLitersPerSecond ~= nil and trailer.fillLitersPerSecond > 0 then
		if fillDelta < (0 - trailer.fillLitersPerSecond) then
			fillDelta = 0 - trailer.fillLitersPerSecond;
		end;
	elseif trailer.fuelFillLitersPerSecond ~= nil and trailer.fuelFillLitersPerSecond > 0 then
		if fillDelta < (0 - trailer.fuelFillLitersPerSecond) then
			fillDelta = 0 - trailer.fuelFillLitersPerSecond;
		end;
	elseif trailer.capacity ~= nil and trailer.capacity > 0 then
		local tmp = 0 - (trailer.capacity / 100);
		if fillDelta < tmp then
			fillDelta = tmp;
		end;
	end;
	if fillDelta < - 120 then
		fillDelta = -120;
	end;
	if fillDeltaOld < fillDelta then
		trailer:setFillLevel(trailer:getFillLevel(fillType) + 0 - (fillDeltaOld - fillDelta),fillType,true);
	end;
	if cFruitsTo ~= nil and cFruitsTo ~= fillType then
		fillType = cFruitsTo;
	end;
	if self.isFarmTrigger then
		local curAmount = g_currentMission:getSiloAmount(fillType);
		if self.stationIdx > 1 then
			curAmount = g_currentMission:getSiloAmount(fType);
		end;
		if curAmount >= mAmount and mAmount > 1 then
			local siloFillType = fillType;
			if fillType == Fillable.FILLTYPE_DRYGRASS then
				siloFillType = Fillable.FILLTYPE_GRASS
			end;
			if self.stationIdx > 1 then
				g_currentMission:setSiloAmount(fType, g_currentMission:getSiloAmount(fType) - fillDelta);
			else
				g_currentMission:setSiloAmount(siloFillType, g_currentMission:getSiloAmount(siloFillType) - fillDelta);
			end;
			if trailer:onEndTip() ~= nil then
				trailer:onEndTip();
			end;
			fillDelta = 0;
		end;
	end;
	if fillDelta < 0 then
	   if self.isFarmTrigger then
		  local siloFillType = fillType;
		  if fillType == Fillable.FILLTYPE_DRYGRASS then
			 siloFillType = Fillable.FILLTYPE_GRASS
		  end;
		  if self.stationIdx > 1 then
		  	g_currentMission:setSiloAmount(fType, g_currentMission:getSiloAmount(fType) - fillDelta);
		  else
		  	g_currentMission:setSiloAmount(siloFillType, g_currentMission:getSiloAmount(siloFillType) - fillDelta);
		  end;
	   else
		  local desc = Fillable.fillTypeIndexToDesc[fillType];
		  local deltaFactor = 1;
		  if Fillable.fillTypeNameToDesc["cow"] == desc then
			deltaFactor = 2001;
		  end;
		  if Fillable.fillTypeNameToDesc["pig"] == desc then
			deltaFactor = 1501;
		  end;
		  if Fillable.fillTypeNameToDesc["sheep"] == desc then
			deltaFactor = 1251;
		  end;
		  if Fillable.fillTypeNameToDesc["chicken"] == desc then
			deltaFactor = 501;
		  end;
		  if self.isBGA == nil then
			self.isBGA = false;
		  end;
		  if not self.isBGA then
			desc.totalAmount = desc.totalAmount - (fillDelta * deltaFactor);
		  end;
		  local priceMultiplier = self.priceMultipliers[fillType];
		  local difficultyMultiplier = math.max(2 * (3 - g_currentMission.missionStats.difficulty), 1); -- 1  2  4
		  local greatDemandMultiplier = 1;
		  local greatDemand = g_currentMission.economyManager:getCurrentGreatDemand(self.stationName, fillType);
		  if greatDemand ~= nil then
			 greatDemandMultiplier = greatDemand.demandMultiplier;
		  end;
		  local money = Fillable.fillTypeIndexToDesc[fillType].pricePerLiter * priceMultiplier * difficultyMultiplier * greatDemandMultiplier * -fillDelta;
		  if Fillable.fillTypeIndexToDesc[fillType].name == "chaff" and Fillable.fillTypeNameToDesc["chaff_fix"] ~= nil then
			money = Fillable.fillTypeNameToDesc["chaff_fix"].pricePerLiter * priceMultiplier * difficultyMultiplier * greatDemandMultiplier * -fillDelta;
			Fillable.fillTypeNameToDesc["chaff_fix"].totalAmount = Fillable.fillTypeNameToDesc["chaff_fix"].totalAmount - (fillDelta * deltaFactor);
		  elseif Fillable.fillTypeIndexToDesc[fillType].name == "silage" and Fillable.fillTypeNameToDesc["silage_fix"] ~= nil then
			money = Fillable.fillTypeNameToDesc["silage_fix"].pricePerLiter * priceMultiplier * difficultyMultiplier * greatDemandMultiplier * -fillDelta;
			Fillable.fillTypeNameToDesc["silage_fix"].totalAmount = Fillable.fillTypeNameToDesc["silage_fix"].totalAmount - (fillDelta * deltaFactor);
		  end;
		  if self.isServer then
			g_currentMission:addSharedMoney(money, "harvestIncome");
		  end;
		  if self.changeType ~= nil then
			if Fillable.fillTypeIndexToDesc[self.changeType] ~= nil then
				Fillable.fillTypeIndexToDesc[self.changeType].totalAmount = Fillable.fillTypeIndexToDesc[self.changeType].totalAmount - (fillDelta * deltaFactor);
			end;
		  end;
	   end;
	   self:updateMoving(-fillDelta);
	end;
end;
TipTrigger.getTipInfoForTrailer = function(self, trailer, tipReferencePointIndex)
	local minDistance, bestPoint = self:getTipDistanceFromTrailer(trailer, tipReferencePointIndex);

	local isAllowed = false;
	if trailer.fillLevel > 0 then
		if self.acceptedFillTypes[trailer.currentFillType] then
			if self.maxAmountAllowed ~= nil and self.maxAmountAllowed[trailer.currentFillType] ~= nil and self.maxAmountAllowed[trailer.currentFillType] > 1 then
				local desc = Fillable.fillTypeIndexToDesc[trailer.currentFillType];
				local fType = trailer.currentFillType;
				if desc.name ~= nil then
					if self.stationIdx > 1 then
						fType = Fillable.fillTypeNameToInt[desc.name..self.stationIdx];
					end;
				end;
				if self.convertFruitsTo == nil then
					self.convertFruitsTo = {};
					self.convertFruitsTo[trailer.currentFillType] = trailer.currentFillType;
				end;
				if self.isFarmTrigger then
					if self.stationIdx > 1 then
						if g_currentMission:getSiloAmount(fType) < self.maxAmountAllowed[trailer.currentFillType] then
							isAllowed = true;
						end;
					else
						if g_currentMission:getSiloAmount(self.convertFruitsTo[trailer.currentFillType]) < self.maxAmountAllowed[trailer.currentFillType] then
							isAllowed = true;
						end;
					end;
				else
					if g_currentMission:getSiloAmount(self.convertFruitsTo[trailer.currentFillType]) < self.maxAmountAllowed[trailer.currentFillType] then
						isAllowed = true;
					end;
				end;
			else
				isAllowed = true;
			end;
		end;
	end;
	return isAllowed, minDistance, bestPoint;
end;
TipTrigger.getNoAllowedText = function(self, trailer)
	if trailer.currentFillType ~= Fillable.FILLTYPE_UNKNOWN and not self.acceptedFillTypes[trailer.currentFillType] then
		return Fillable.fillTypeIndexToDesc[trailer.currentFillType].nameI18N .. g_i18n:getText("notAcceptedHere");
	else
		local mAA = 0;
		if self.maxAmountAllowed ~= nil and self.maxAmountAllowed[fillType] ~= nil and self.maxAmountAllowed[fillType] > 1 then
			mAA = self.maxAmountAllowed[trailer.currentFillType];
		end;
		local desc = Fillable.fillTypeIndexToDesc[trailer.currentFillType];
		local fType = trailer.currentFillType;
		if self.convertFruitsTo == nil then
			self.convertFruitsTo = {};
			self.convertFruitsTo[trailer.currentFillType] = trailer.currentFillType;
		end;
		local gSA = g_currentMission:getSiloAmount(self.convertFruitsTo[trailer.currentFillType]);
		if desc ~= nil and desc.name ~= nil then
			if self.stationIdx > 1 then
				fType = Fillable.fillTypeNameToInt[desc.name..self.stationIdx];
				gSA = g_currentMission:getSiloAmount(fType);
			end;
		end;
		if mAA ~= nil and gSA ~= nil then
			if mAA > 1 and gSA >= mAA then
				return string.format(g_i18n:getText("notEnoughSpaceForTipping"),Fillable.fillTypeIndexToDesc[trailer.currentFillType].nameI18N);
			end;
		end;
	end;
	return nil;
end;
